@@ -107,7 +107,7 @@ sub utgcns ($$$) {
     print F "  -e " . getGlobal("cnsErrorRate") . " \\\n";
     print F "  -quick \\\n"      if (getGlobal("cnsConsensus") eq "quick");
     print F "  -pbdagcon \\\n"   if (getGlobal("cnsConsensus") eq "pbdagcon");
-    print F "  -edlib    \\\n"   if (getGlobal("canuIteration") > 0);
+    print F "  -edlib    \\\n"   if (getGlobal("canuIteration") >= 0);
     print F "  -utgcns \\\n"     if (getGlobal("cnsConsensus") eq "utgcns");
     print F "  -threads " . getGlobal("cnsThreads") . " \\\n";
     print F "&& \\\n";
@@ -117,7 +117,7 @@ sub utgcns ($$$) {
     print F "\n";
     print F "exit 0\n";
 
-    if (getGlobal("canuIteration") == 0) {
+    if (getGlobal("canuIteration") < 0) {
         print STDERR "-- Using fast alignment for consensus (iteration '", getGlobal("canuIteration"), "').\n";
     } else {
         print STDERR "-- Using slow alignment for consensus (iteration '", getGlobal("canuIteration"), "').\n";
@@ -125,6 +125,7 @@ sub utgcns ($$$) {
 
     close(F);
 
+    makeExecutable("$path/consensus.sh");
     stashFile("$path/consensus.sh");
 }
 
@@ -143,9 +144,7 @@ sub cleanupPartitions ($$) {
 
     print STDERR "-- Partitioned gkpStore is older than tigs, rebuild partitioning (gkpStore $gkpTime days old; ctgStore $tigTime days old).\n";
 
-    if (runCommandSilently("unitigging", "rm -rf ./$asm.${tag}Store/partitionedReads.gkpStore", 1)) {
-        caExit("failed to remove old partitions (unitigging/$asm.${tag}Store/partitionedReads.gkpStore/partitions), can't continue until these are removed", undef);
-    }
+    remove_tree("unitigging/$asm.${tag}Store/partitionedReads.gkpStore");
 }
 
 
@@ -327,19 +326,21 @@ sub consensusCheck ($) {
 
     if (scalar(@failedJobs) > 0) {
 
-        #  If not the first attempt, report the jobs that failed, and that we're recomputing.
-
-        if ($attempt > 1) {
-            print STDERR "--\n";
-            print STDERR "-- ", scalar(@failedJobs), " consensus jobs failed:\n";
-            print STDERR $failureMessage;
-            print STDERR "--\n";
-        }
-
         #  If too many attempts, give up.
 
-        if ($attempt > getGlobal("canuIterationMax")) {
-            caExit("failed to generate consensus.  Made " . ($attempt-1) . " attempts, jobs still failed", undef);
+        if ($attempt >= getGlobal("canuIterationMax")) {
+            print STDERR "--\n";
+            print STDERR "-- Consensus jobs failed, tried $attempt times, giving up.\n";
+            print STDERR $failureMessage;
+            print STDERR "--\n";
+            caExit(undef, undef);
+        }
+
+        if ($attempt > 0) {
+            print STDERR "--\n";
+            print STDERR "-- Consensus jobs failed, retry.\n";
+            print STDERR $failureMessage;
+            print STDERR "--\n";
         }
 
         #  Otherwise, run some jobs.
@@ -374,13 +375,20 @@ sub consensusCheck ($) {
 
 
 
-sub purgeFiles ($$$$$) {
+sub purgeFiles ($$$$$$) {
+    my $asm     = shift @_;
     my $tag     = shift @_;
     my $Ncns    = shift @_;
     my $Nfastq  = shift @_;
     my $Nlayout = shift @_;
     my $Nlog    = shift @_;
 
+    remove_tree("unitigging/$asm.ctgStore/partitionedReads.gkpStore");  #  The partitioned gkpStores
+    remove_tree("unitigging/$asm.utgStore/partitionedReads.gkpStore");  #  are useless now.  Bye bye!
+
+    unlink "unitigging/$asm.ctgStore/partitionedReads.log";
+    unlink "unitigging/$asm.utgStore/partitionedReads.log";
+
     my $path = "unitigging/5-consensus";
 
     open(F, "< $path/$tag.files") or caExit("can't open '$path/$tag.files' for reading: $!\n", undef);
@@ -391,25 +399,25 @@ sub purgeFiles ($$$$$) {
             my $ID4 = substr("000"   . $2, -4);
             my $ID0 = $2;
 
-            if (-e "$1/$ID4.cns") {
+            if (-e "unitigging/$1/$ID4.cns") {
                 $Ncns++;
-                unlink "$1/$ID4.cns";
+                unlink "unitigging/$1/$ID4.cns";
             }
-            if (-e "$1/$ID4.fastq") {
+            if (-e "unitigging/$1/$ID4.fastq") {
                 $Nfastq++;
-                unlink "$1/$ID4.fastq";
+                unlink "unitigging/$1/$ID4.fastq";
             }
-            if (-e "$1/$ID4.layout") {
+            if (-e "unitigging/$1/$ID4.layout") {
                 $Nlayout++;
-                unlink "$1/$ID4.layout";
+                unlink "unitigging/$1/$ID4.layout";
             }
-            if (-e "$1/consensus.$ID6.out") {
+            if (-e "unitigging/$1/consensus.$ID6.out") {
                 $Nlog++;
-                unlink "$1/consensus.$ID6.out";
+                unlink "unitigging/$1/consensus.$ID6.out";
             }
-            if (-e "$1/consensus.$ID0.out") {
+            if (-e "unitigging/$1/consensus.$ID0.out") {
                 $Nlog++;
-                unlink "$1/consensus.$ID0.out";
+                unlink "unitigging/$1/consensus.$ID0.out";
             }
 
         } else {
@@ -418,6 +426,9 @@ sub purgeFiles ($$$$$) {
     }
     close(F);
 
+    unlink "$path/$tag.files";
+    rmdir  "$path/$tag";
+
     return($Ncns, $Nfastq, $Nlayout, $Nlog);
 }
 
@@ -463,6 +474,7 @@ sub consensusLoad ($) {
         if (runCommand("unitigging", $cmd)) {
             caExit("failed to load unitig consensus into ctgStore", "$path/ctgcns.files.ctgStoreLoad.err");
         }
+        unlink "$path/ctgcns.files.ctgStoreLoad.err";
 
         stashFile("unitigging/$asm.ctgStore/seqDB.v002.dat");
         stashFile("unitigging/$asm.ctgStore/seqDB.v002.tig");
@@ -488,6 +500,7 @@ sub consensusLoad ($) {
         if (runCommand("unitigging", $cmd)) {
             caExit("failed to load unitig consensus into utgStore", "$path/utgcns.files.utgStoreLoad.err");
         }
+        unlink "$path/utgcns.files.utgStoreLoad.err";
 
         stashFile("unitigging/$asm.utgStore/seqDB.v002.dat");
         stashFile("unitigging/$asm.utgStore/seqDB.v002.tig");
@@ -504,8 +517,8 @@ sub consensusLoad ($) {
         my $Nlayout = 0;
         my $Nlog    = 0;
 
-        ($Ncns, $Nfastq, $Nlayout, $Nlog) = purgeFiles("ctgcns", $Ncns, $Nfastq, $Nlayout, $Nlog);
-        ($Ncns, $Nfastq, $Nlayout, $Nlog) = purgeFiles("utgcns", $Ncns, $Nfastq, $Nlayout, $Nlog);
+        ($Ncns, $Nfastq, $Nlayout, $Nlog) = purgeFiles($asm, "ctgcns", $Ncns, $Nfastq, $Nlayout, $Nlog);
+        ($Ncns, $Nfastq, $Nlayout, $Nlog) = purgeFiles($asm, "utgcns", $Ncns, $Nfastq, $Nlayout, $Nlog);
 
         print STDERR "-- Purged $Ncns .cns outputs.\n"        if ($Ncns > 0);
         print STDERR "-- Purged $Nfastq .fastq outputs.\n"    if ($Nfastq > 0);
@@ -580,7 +593,11 @@ sub alignGFA ($) {
 
     goto allDone   if (skipStage($asm, "alignGFA") == 1);
     goto allDone   if (fileExists("unitigging/4-unitigger/$asm.contigs.aligned.gfa") &&
-                       fileExists("unitigging/4-unitigger/$asm.unitigs.aligned.gfa"));
+                       fileExists("unitigging/4-unitigger/$asm.unitigs.aligned.gfa") &&
+                       fileExists("unitigging/4-unitigger/$asm.unitigs.aligned.bed"));
+
+    #  If a large genome, run this on the grid, else, run in the canu process itself.
+    my $runGrid = (getGlobal("genomeSize") >= 40000000);
 
     fetchFile("$path/alignGFA.sh");
 
@@ -590,7 +607,7 @@ sub alignGFA ($) {
         print F "\n";
         print F getBinDirectoryShellCode();
         print F "\n";
-        print F setWorkDirectoryShellCode($path);
+        print F setWorkDirectoryShellCode($path)   if ($runGrid);   #  If not local, need to cd first.
         print F "\n";
         print F fetchFileShellCode("unitigging/$asm.utgStore", "seqDB.v001.dat", "");
         print F fetchFileShellCode("unitigging/$asm.utgStore", "seqDB.v001.tig", "");
@@ -605,6 +622,7 @@ sub alignGFA ($) {
         print F fetchFileShellCode("unitigging/$asm.ctgStore", "seqDB.v002.tig", "");
         print F "\n";
         print F "\n";
+
         print F "if [ ! -e ./$asm.unitigs.aligned.gfa ] ; then\n";
         print F "  \$bin/alignGFA \\\n";
         print F "    -T ../$asm.utgStore 2 \\\n";
@@ -617,6 +635,7 @@ sub alignGFA ($) {
         print F "fi\n";
         print F "\n";
         print F "\n";
+
         print F "if [ ! -e ./$asm.contigs.aligned.gfa ] ; then\n";
         print F "  \$bin/alignGFA \\\n";
         print F "    -T ../$asm.ctgStore 2 \\\n";
@@ -629,8 +648,24 @@ sub alignGFA ($) {
         print F "fi\n";
         print F "\n";
         print F "\n";
+
+        print F "if [ ! -e ./$asm.unitigs.aligned.bed ] ; then\n";
+        print F "  \$bin/alignGFA -bed \\\n";
+        print F "    -T ../$asm.utgStore 2 \\\n";
+        print F "    -C ../$asm.ctgStore 2 \\\n";
+        print F "    -i ./$asm.unitigs.bed \\\n";
+        print F "    -o ./$asm.unitigs.aligned.bed \\\n";
+        print F "    -t " . getGlobal("gfaThreads") . " \\\n";
+        print F "  > ./$asm.unitigs.aligned.bed.err 2>&1";
+        print F "\n";
+        print F stashFileShellCode("$path", "$asm.unitigs.aligned.bed", "  ");
+        print F "fi\n";
+        print F "\n";
+        print F "\n";
+
         print F "if [ -e ./$asm.unitigs.aligned.gfa -a \\\n";
-        print F "     -e ./$asm.contigs.aligned.gfa ] ; then\n";
+        print F "     -e ./$asm.contigs.aligned.gfa -a \\\n";
+        print F "     -e ./$asm.unitigs.aligned.bed ] ; then\n";
         print F "  echo GFA alignments updated.\n";
         print F "  exit 0\n";
         print F "else\n";
@@ -639,8 +674,7 @@ sub alignGFA ($) {
         print F "fi\n";
         close(F);
 
-        system("chmod +x $path/alignGFA.sh");
-
+        makeExecutable("$path/alignGFA.sh");
         stashFile("$path/alignGFA.sh");
     }
 
@@ -648,31 +682,31 @@ sub alignGFA ($) {
     #  shows how to process multiple jobs.  This only checks for the existence of the final outputs.
     #  (meryl and unitig are the same)
 
-    #  If not the first attempt, report the jobs that failed, and that we're recomputing.
-
-    if ($attempt > 1) {
-        print STDERR "--\n";
-        print STDERR "-- GFA alignment failed.\n";
-        print STDERR "--\n";
-    }
-
     #  If too many attempts, give up.
 
-    if ($attempt > getGlobal("canuIterationMax")) {
-        caExit("failed to align GFA links.  Made " . ($attempt-1) . " attempts, jobs still failed", undef);
+    if ($attempt >= getGlobal("canuIterationMax")) {
+        print STDERR "--\n";
+        print STDERR "-- Graph alignment jobs failed, tried $attempt times, giving up.\n";
+        print STDERR "--\n";
+        caExit(undef, undef);
     }
 
-    #  Otherwise, run some jobs.  If the genome is small, just do it here and now, otherwise,
-    #  run on the grid.
+    if ($attempt > 0) {
+        print STDERR "--\n";
+        print STDERR "-- Graph alignment jobs failed, retry.\n";
+        print STDERR "--\n";
+    }
+
+    #  Otherwise, run some jobs.
 
     emitStage($asm, "alignGFA", $attempt);
 
-    if (getGlobal("genomeSize") < 40000000) {
-        if (runCommand("$path", "./alignGFA.sh")) {
+    if ($runGrid) {
+        submitOrRunParallelJob($asm, "gfa", $path, "alignGFA", (1));
+    } else {
+        if (runCommand($path, "./alignGFA.sh")) {
             caExit("failed to align contigs", "./$asm.contigs.aligned.gfa.err");
         }
-    } else {
-        submitOrRunParallelJob($asm, "gfa", $path, "alignGFA", (1));
     }
 
     return;
