@@ -59,6 +59,7 @@ require Exporter;
              skipStage
              emitStage
              touch
+             makeExecutable
              getInstallDirectory
              getJobIDShellCode
              getLimitShellCode
@@ -167,8 +168,9 @@ sub schedulerRun () {
     }
 }
 
-sub schedulerFinish ($) {
+sub schedulerFinish ($$) {
     my $dir = shift @_;
+    my $nam = shift @_;
     my $child;
     my @newProcesses;
     my $remain;
@@ -179,8 +181,8 @@ sub schedulerFinish ($) {
     my $diskfree  = (defined($dir)) ? (diskSpace($dir)) : (0);
 
     print STDERR "----------------------------------------\n";
-    print STDERR "-- Starting concurrent execution on ", scalar(localtime()), " with $diskfree GB free disk space ($remain processes; $numberOfProcesses concurrently)\n"  if  (defined($dir));
-    print STDERR "-- Starting concurrent execution on ", scalar(localtime()), " ($remain processes; $numberOfProcesses concurrently)\n"                                    if (!defined($dir));
+    print STDERR "-- Starting '$nam' concurrent execution on ", scalar(localtime()), " with $diskfree GB free disk space ($remain processes; $numberOfProcesses concurrently)\n"  if  (defined($dir));
+    print STDERR "-- Starting '$nam' concurrent execution on ", scalar(localtime()), " ($remain processes; $numberOfProcesses concurrently)\n"                                    if (!defined($dir));
     print STDERR "\n";
     print STDERR "    cd $dir\n";
 
@@ -242,6 +244,13 @@ sub touch ($@) {
 
 
 
+sub makeExecutable ($) {
+    my $file = shift @_;
+
+    chmod(0755 & ~umask(), $file);
+}
+
+
 #
 #  State management
 #
@@ -346,41 +355,34 @@ sub getJobIDShellCode () {
 #  Emits a block of shell code to change shell imposed limit on the number of open files and
 #  processes.
 #
-sub getLimitShellCode ($) {
-    my $which = shift @_;
+sub getLimitShellCode () {
     my $string;
 
-    if ($which eq "processes") {
-        $string .= "\n";
-        $string .= "max=`ulimit -Hu`\n";
-        $string .= "bef=`ulimit -Su`\n";
-        $string .= "if [ \$bef -lt \$max ] ; then\n";
-        $string .= "  ulimit -Su \$max\n";
-        $string .= "  aft=`ulimit -Su`\n";
-        $string .= "  echo \"Changed max processes per user from \$bef to \$aft (max \$max).\"\n";
-        $string .= "  echo \"\"\n";
-        $string .= "else\n";
-        $string .= "  echo \"Max processes per user limited to \$bef, no increase possible.\"\n";
-        $string .= "  echo \"\"\n";
-        $string .= "fi\n";
-        $string .= "\n";
-    }
-
-    if ($which eq "files") {
-        $string .= "\n";
-        $string .= "max=`ulimit -Hn`\n";
-        $string .= "bef=`ulimit -Sn`\n";
-        $string .= "if [ \$bef -lt \$max ] ; then\n";
-        $string .= "  ulimit -Sn \$max\n";
-        $string .= "  aft=`ulimit -Sn`\n";
-        $string .= "  echo \"Changed max open files from \$bef to \$aft (max \$max).\"\n";
-        $string .= "  echo \"\"\n";
-        $string .= "else\n";
-        $string .= "  echo \"Max open files limited to \$bef, no increase possible.\"\n";
-        $string .= "  echo \"\"\n";
-        $string .= "fi\n";
-        $string .= "\n";
-    }
+    $string .= "echo \"\"\n";
+    $string .= "echo \"Attempting to increase maximum allowed processes and open files.\"";
+    $string .= "\n";
+    $string .= "max=`ulimit -Hu`\n";
+    $string .= "bef=`ulimit -Su`\n";
+    $string .= "if [ \$bef -lt \$max ] ; then\n";
+    $string .= "  ulimit -Su \$max\n";
+    $string .= "  aft=`ulimit -Su`\n";
+    $string .= "  echo \"  Changed max processes per user from \$bef to \$aft (max \$max).\"\n";
+    $string .= "else\n";
+    $string .= "  echo \"  Max processes per user limited to \$bef, no increase possible.\"\n";
+    $string .= "fi\n";
+    $string .= "\n";
+    $string .= "max=`ulimit -Hn`\n";
+    $string .= "bef=`ulimit -Sn`\n";
+    $string .= "if [ \$bef -lt \$max ] ; then\n";
+    $string .= "  ulimit -Sn \$max\n";
+    $string .= "  aft=`ulimit -Sn`\n";
+    $string .= "  echo \"  Changed max open files from \$bef to \$aft (max \$max).\"\n";
+    $string .= "else\n";
+    $string .= "  echo \"  Max open files limited to \$bef, no increase possible.\"\n";
+    $string .= "fi\n";
+    $string .= "\n";
+    $string .= "echo \"\"\n";
+    $string .= "\n";
 
     return($string);
 }
@@ -663,7 +665,7 @@ sub submitScript ($$) {
     print F "\$bin/canu " . getCommandLineOptions() . " canuIteration=" . getGlobal("canuIteration") . "\n";
     close(F);
 
-    system("chmod +x $script");
+    makeExecutable("$script");
 
     #  Construct a submission command line.
 
@@ -844,6 +846,18 @@ sub buildThreadOption ($) {
 }
 
 
+sub purgeGridJobSubmitScripts ($$) {
+    my $path    = shift @_;
+    my $script  = shift @_;
+    my $idx     = "01";
+
+    while (-e "$path/$script.jobSubmit-$idx.sh") {
+        unlink "$path/$script.jobSubmit-$idx.sh";
+        $idx++;
+    }
+}
+
+
 sub buildGridJob ($$$$$$$$$) {
     my $asm     = shift @_;
     my $jobType = shift @_;
@@ -894,12 +908,18 @@ sub buildGridJob ($$$$$$$$$) {
     $opts .= "$outputOption "   if (defined($outputOption));
     $opts =~ s/\s+$//;
 
+    #  Find a unique file name to save the command.
+
+    my $idx = "01";
+
+    while (-e "$path/$script.jobSubmit-$idx.sh") {
+        $idx++;
+    }
+
     #  Build and save the command line.  Return the command PREFIX (we'll be adding .sh and .out as
     #  appropriate), and the job name it will be submitted with (which isn't expected to be used).
 
-    my $cmd;
-
-    open(F, "> $path/$script.jobSubmit.sh") or die;
+    open(F, "> $path/$script.jobSubmit-$idx.sh") or die;
     print F "#!/bin/sh\n";
     print F "\n";
     print F "$submitCommand \\\n";
@@ -907,12 +927,12 @@ sub buildGridJob ($$$$$$$$$) {
     print F "  $nameOption \"$jobName\" \\\n";
     print F "  $arrayOpt \\\n";
     print F "  ./$script.sh $arrayOff \\\n";
-    print F "> ./$script.jobSubmit.out 2>&1\n";
+    print F "> ./$script.jobSubmit-$idx.out 2>&1\n";
     close(F);
 
-    chmod 0755, "$path/$script.jobSubmit.sh";
+    makeExecutable("$path/$script.jobSubmit-$idx.sh");
 
-    return("$script.jobSubmit", $jobName);
+    return("$script.jobSubmit-$idx", $jobName);
 }
 
 
@@ -1029,7 +1049,7 @@ sub submitOrRunParallelJob ($$$$@) {
 
     #  The script MUST be executable.
 
-    system("chmod +x \"$path/$script.sh\"");
+    makeExecutable("$path/$script.sh");
 
     #  Report what we're doing.
 
@@ -1091,6 +1111,8 @@ sub submitOrRunParallelJob ($$$$@) {
 
         print STDERR "--\n";
 
+        purgeGridJobSubmitScripts($path, $script);
+
         foreach my $j (@jobs) {
             my ($cmd, $jobName) = buildGridJob($asm, $jobType, $path, $script, $mem, $thr, $dsk, $j, undef);
 
@@ -1201,6 +1223,8 @@ sub submitOrRunParallelJob ($$$$@) {
         print STDERR "Please run the following commands to submit jobs to the grid for execution using $mem gigabytes memory and $thr threads:\n";
         print STDERR "\n";
 
+        purgeGridJobSubmitScripts($path, $script);
+
         foreach my $j (@jobs) {
             my  $cwd = getcwd();
             my ($cmd, $jobName) = buildGridJob($asm, $jobType, $path, $script, $mem, $thr, $dsk, $j, undef);
@@ -1247,7 +1271,7 @@ sub submitOrRunParallelJob ($$$$@) {
     my $nParallel  = $nCParallel < $nMParallel ? $nCParallel : $nMParallel;
 
     schedulerSetNumberOfProcesses($nParallel);
-    schedulerFinish($path);
+    schedulerFinish($path, $jobType);
 }
 
 
@@ -1430,34 +1454,40 @@ sub findExecutable ($) {
 
 #  Use caExit() for transient errors, like not opening files, processes that die, etc.
 sub caExit ($$) {
-    my  $asm   = getGlobal("onExitNam");
-    my  $msg   = shift @_;
-    my  $log   = shift @_;
+    my  $asm     = getGlobal("onExitNam");
+    my  $msg     = shift @_;
+    my  $log     = shift @_;
+    my  $version = getGlobal("version");
 
-    print STDERR "================================================================================\n";
-    print STDERR "Don't panic, but a mostly harmless error occurred and Canu stopped.\n";
     print STDERR "\n";
+    print STDERR "ABORT:\n";
+    print STDERR "ABORT: $version\n";
+    print STDERR "ABORT: Don't panic, but a mostly harmless error occurred and Canu stopped.\n";
+    print STDERR "ABORT: Try restarting.  If that doesn't work, ask for help.\n";
+    print STDERR "ABORT:\n";
+    print STDERR "ABORT:   $msg.\n"     if (defined($msg));
+    print STDERR "ABORT:\n"             if (defined($msg));
 
-    if (defined($log)) {
+    if (defined($log) && -e $log) {
         my  $df = diskSpace($log);
 
-        print STDERR "Disk space available:  $df GB\n";
-        print STDERR "\n";
+        print STDERR "ABORT: Disk space available:  $df GB\n";
+        print STDERR "ABORT:\n";
     }
 
     if (-e $log) {
-        print STDERR "Last 50 lines of the relevant log file ($log):\n";
-        print STDERR "\n";
-        system("tail -n 50 $log");
-        print STDERR "\n";
+        print STDERR "ABORT: Last 50 lines of the relevant log file ($log):\n";
+        print STDERR "ABORT:\n";
+
+        open(Z, "tail -n 50 $log");
+        while (<Z>) {
+            print STDERR "ABORT: $_";
+        }
+        close(Z);
+
+        print STDERR "ABORT:\n";
     }
 
-    my $version = getGlobal("version");
-
-    print STDERR "$version failed with:\n";
-    print STDERR "  $msg\n";
-    print STDERR "\n";
-
     my $fail = getGlobal('onFailure');
     if (defined($fail)) {
         runCommandSilently(getGlobal("onExitDir"), "$fail $asm", 0);
@@ -1473,26 +1503,35 @@ sub caFailure ($$) {
     my  $msg     = shift @_;
     my  $log     = shift @_;
     my  $version = getGlobal("version");
-    my  $trace   = longmess(undef);
+    my  $trace   = longmess("Failed");
+
+    $trace =~ s/\n/\nCRASH: /g;
 
-    print STDERR "================================================================================\n";
-    print STDERR "Please panic.  Canu failed, and it shouldn't have.\n";
-    print STDERR "\n";
-    print STDERR "Stack trace:\n";
-    print STDERR "\n";
-    print STDERR "$trace\n";
     print STDERR "\n";
+    print STDERR "CRASH:\n";
+    print STDERR "CRASH: $version\n";
+    print STDERR "CRASH: Please panic, this is abnormal.\n";
+    print STDERR "ABORT:\n";
+    print STDERR "CRASH:   $msg.\n";
+    print STDERR "CRASH:\n";
+    print STDERR "CRASH: $trace\n";
+    #print STDERR "CRASH:\n";   #  $trace has an extra CRASH: at the end
 
     if (-e $log) {
-        print STDERR "Last few lines of the relevant log file ($log):\n";
-        print STDERR "\n";
-        system("tail -n 50 $log");
-    }
+        print STDERR "CRASH: Last 50 lines of the relevant log file ($log):\n";
+        print STDERR "CRASH:\n";
 
-    print STDERR "\n";
-    print STDERR "$version failed with:\n";
-    print STDERR "  $msg\n";
-    print STDERR "\n";
+        open(Z, "tail -n 50 $log");
+        while (<Z>) {
+            print STDERR "CRASH: $_";
+        }
+        close(Z);
+
+        print STDERR "CRASH:\n";
+    } else {
+        print STDERR "CRASH: No log file supplied.\n";
+        print STDERR "CRASH:\n";
+    }
 
     my $fail = getGlobal('onFailure');
     if (defined($fail)) {
