@@ -174,6 +174,13 @@ sub generateOutputs ($) {
         stashFile("$asm.unitigs.gfa");
     }
 
+    if ((! fileExists("$asm.unitigs.bed")) &&
+        (  fileExists("unitigging/4-unitigger/$asm.unitigs.aligned.bed"))) {
+        fetchFile("unitigging/4-unitigger/$asm.unitigs.aligned.bed");
+        copy("unitigging/4-unitigger/$asm.unitigs.aligned.bed", "$asm.unitigs.bed");
+        stashFile("$asm.unitigs.bed");
+    }
+
     #  User-supplied termination command.
 
     if (defined(getGlobal("onSuccess"))) {
