@@ -232,8 +232,7 @@ sub overlapConfigure ($$$) {
         print F "exit 0\n";
         close(F);
 
-        system("chmod +x $path/overlap.sh");
-
+        makeExecutable("$path/overlap.sh");
         stashFile("$path/overlap.sh");
     }
 
@@ -391,7 +390,7 @@ sub overlapCheck ($$$) {
                 push @miscJobs,    "1-overlapper/$1.counts\n";
 
             } else {
-                $failureMessage .= "--   job 1-overlapper/$1.ovb FAILED.\n";
+                $failureMessage .= "--   job $path/$1.ovb FAILED.\n";
                 push @failedJobs, $currentJobID;
             }
 
@@ -405,19 +404,21 @@ sub overlapCheck ($$$) {
 
     if (scalar(@failedJobs) > 0) {
 
-        #  If not the first attempt, report the jobs that failed, and that we're recomputing.
-
-        if ($attempt > 1) {
-            print STDERR "--\n";
-            print STDERR "-- ", scalar(@failedJobs), " overlapper jobs failed:\n";
-            print STDERR $failureMessage;
-            print STDERR "--\n";
-        }
-
         #  If too many attempts, give up.
 
-        if ($attempt > getGlobal("canuIterationMax")) {
-            caExit("failed to overlap.  Made " . ($attempt-1) . " attempts, jobs still failed", undef);
+        if ($attempt >= getGlobal("canuIterationMax")) {
+            print STDERR "--\n";
+            print STDERR "-- Overlap jobs failed, tried $attempt times, giving up.\n";
+            print STDERR $failureMessage;
+            print STDERR "--\n";
+            caExit(undef, undef);
+        }
+
+        if ($attempt > 0) {
+            print STDERR "--\n";
+            print STDERR "-- Overlap jobs failed, retry.\n";
+            print STDERR $failureMessage;
+            print STDERR "--\n";
         }
 
         #  Otherwise, run some jobs.
